@@ -15,8 +15,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GDK_DND_PRIVATE_H__
-#define __GDK_DND_PRIVATE_H__
+#pragma once
 
 #include "gdkdrag.h"
 
@@ -35,8 +34,8 @@ struct _GdkDragClass {
 
   GdkSurface*  (*get_drag_surface) (GdkDrag *drag);
   void        (*set_hotspot)   (GdkDrag  *drag,
-                                gint             hot_x,
-                                gint             hot_y);
+                                int              hot_x,
+                                int              hot_y);
   void        (*drop_done)     (GdkDrag   *drag,
                                 gboolean          success);
 
@@ -83,4 +82,3 @@ gboolean gdk_drag_write_finish        (GdkDrag             *drag,
 
 G_END_DECLS
 
-#endif
