@@ -96,8 +96,6 @@ struct _GdkSurface
   GSList *draw_contexts;
   GdkDrawContext *paint_context;
 
-  cairo_region_t *opaque_region;
-
   GdkSeat *current_shortcuts_inhibited_seat;
 
   GPtrArray *subsurfaces;
@@ -156,6 +154,11 @@ struct _GdkSurfaceClass
                                          double              dy);
 
   double       (* get_scale)              (GdkSurface      *surface);
+  /* see docs for gdk_draw_context_get_buffer_size() */
+  void         (* get_buffer_size)        (GdkSurface      *surface,
+                                           GdkDrawContext  *context,
+                                           guint           *out_width,
+                                           guint           *out_height);
 
   void         (* set_opaque_region)      (GdkSurface      *surface,
                                            cairo_region_t *region);
@@ -253,14 +256,17 @@ gdk_gravity_flip_vertically (GdkGravity anchor)
   g_assert_not_reached ();
 }
 
-void       _gdk_surface_destroy           (GdkSurface      *surface,
-                                           gboolean        foreign_destroy);
-void       gdk_surface_invalidate_rect    (GdkSurface           *surface,
-                                           const GdkRectangle   *rect);
-void       gdk_surface_invalidate_region  (GdkSurface           *surface,
-                                           const cairo_region_t *region);
-void       _gdk_surface_clear_update_area (GdkSurface      *surface);
-void       _gdk_surface_update_size       (GdkSurface      *surface);
+void       _gdk_surface_destroy           (GdkSurface           *surface,
+                                           gboolean               foreign_destroy);
+void       gdk_surface_invalidate_rect    (GdkSurface            *surface,
+                                           const GdkRectangle    *rect);
+void       gdk_surface_invalidate_region  (GdkSurface            *surface,
+                                           const cairo_region_t  *region);
+void       _gdk_surface_clear_update_area (GdkSurface            *surface);
+void       _gdk_surface_update_size       (GdkSurface            *surface);
+void       gdk_surface_set_opaque_rect    (GdkSurface            *self,
+                                           const graphene_rect_t *rect);
+gboolean   gdk_surface_is_opaque          (GdkSurface            *self);
 
 GdkGLContext * gdk_surface_get_paint_gl_context (GdkSurface *surface,
                                                  GError   **error);
