@@ -110,6 +110,12 @@ gdk_wayland_clipboard_data_source_send (void                  *data,
                      source, mime_type, fd);
 
   mime_type = gdk_intern_mime_type (mime_type);
+  if (!mime_type)
+    {
+      close (fd);
+      return;
+    }
+
   stream = g_unix_output_stream_new (fd, TRUE);
 
   gdk_clipboard_write_async (GDK_CLIPBOARD (cb),
@@ -225,19 +231,18 @@ gdk_wayland_clipboard_read_async (GdkClipboard        *clipboard,
   g_task_set_priority (task, io_priority);
   g_task_set_source_tag (task, gdk_wayland_clipboard_read_async);
 
-#ifdef G_ENABLE_DEBUG
   if (GDK_DISPLAY_DEBUG_CHECK (gdk_clipboard_get_display (GDK_CLIPBOARD (cb)), CLIPBOARD))
     {
       char *s = gdk_content_formats_to_string (formats);
       gdk_debug_message ("%p: read for %s", cb, s);
       g_free (s);
     }
-#endif
   mime_type = gdk_content_formats_match_mime_type (formats, cb->offer_formats);
   if (mime_type == NULL)
     {
       g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
                                _("No compatible transfer format found"));
+      g_object_unref (task);
       return;
     }
   /* offer formats should be empty if we have no offer */
@@ -248,6 +253,7 @@ gdk_wayland_clipboard_read_async (GdkClipboard        *clipboard,
   if (!g_unix_open_pipe (pipe_fd, O_CLOEXEC, &error))
     {
       g_task_return_error (task, error);
+      g_object_unref (task);
       return;
     }
 
@@ -255,6 +261,7 @@ gdk_wayland_clipboard_read_async (GdkClipboard        *clipboard,
   stream = g_unix_input_stream_new (pipe_fd[0], TRUE);
   close (pipe_fd[1]);
   g_task_return_pointer (task, stream, g_object_unref);
+  g_object_unref (task);
 }
 
 static GInputStream *
@@ -322,14 +329,12 @@ gdk_wayland_clipboard_claim_remote (GdkWaylandClipboard  *cb,
 
   gdk_wayland_clipboard_discard_offer (cb);
 
-#ifdef G_ENABLE_DEBUG
   if (GDK_DISPLAY_DEBUG_CHECK (gdk_clipboard_get_display (GDK_CLIPBOARD (cb)), CLIPBOARD))
     {
       char *s = gdk_content_formats_to_string (formats);
       gdk_debug_message ("%p: remote clipboard claim for %s", cb, s);
       g_free (s);
     }
-#endif
   cb->offer_formats = formats;
   cb->offer = offer;
 
