@@ -24,14 +24,10 @@
 #include <errno.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <sys/sysmacros.h>
-
-#ifdef HAVE_LINUX_MEMFD_H
-#include <linux/memfd.h>
-#endif
 
+#ifdef HAVE_SYS_MMAN_H
 #include <sys/mman.h>
-#include <sys/syscall.h>
+#endif
 
 #include <glib.h>
 #include <gio/gio.h>
@@ -62,7 +58,7 @@
 #include <wayland/server-decoration-client-protocol.h>
 #include "linux-dmabuf-unstable-v1-client-protocol.h"
 #include "presentation-time-client-protocol.h"
-#include "xx-color-management-v4-client-protocol.h"
+#include "color-management-v1-client-protocol.h"
 
 #include "wm-button-layout-translation.h"
 
@@ -97,7 +93,7 @@
 
 #define MIN_SYSTEM_BELL_DELAY_MS 20
 
-#define GTK_SHELL1_VERSION       5
+#define GTK_SHELL1_VERSION       6
 #define OUTPUT_VERSION_WITH_DONE 2
 #define NO_XDG_OUTPUT_DONE_SINCE_VERSION 3
 #define OUTPUT_VERSION           3
@@ -536,8 +532,8 @@ gdk_registry_handle_global (void               *data,
                           &wp_presentation_interface,
                           MIN (version, 1));
     }
-  else if (strcmp (interface, xx_color_manager_v4_interface.name) == 0 &&
-           gdk_has_feature (GDK_FEATURE_COLOR_MANAGEMENT))
+  else if (strcmp (interface, wp_color_manager_v1_interface.name) == 0 &&
+           GDK_DISPLAY_DEBUG_CHECK (GDK_DISPLAY (display_wayland), COLOR_MANAGEMENT))
     {
       display_wayland->color = gdk_wayland_color_new (display_wayland, registry, id, version);
     }
@@ -554,6 +550,12 @@ gdk_registry_handle_global (void               *data,
         wl_registry_bind (display_wayland->wl_registry, id,
                           &xdg_system_bell_v1_interface, 1);
     }
+  else if (strcmp (interface, wp_cursor_shape_manager_v1_interface.name) == 0)
+    {
+      display_wayland->cursor_shape =
+        wl_registry_bind (display_wayland->wl_registry, id,
+                          &wp_cursor_shape_manager_v1_interface, 1);
+    }
 
   g_hash_table_insert (display_wayland->known_globals,
                        GUINT_TO_POINTER (id), g_strdup (interface));
@@ -1009,7 +1011,7 @@ gdk_wayland_display_get_monitors (GdkDisplay *display)
 
 static GdkMonitor *
 gdk_wayland_display_get_monitor_at_surface (GdkDisplay *display,
-                                           GdkSurface  *window)
+                                            GdkSurface *window)
 {
   GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
   struct wl_output *output;
@@ -1142,7 +1144,7 @@ get_cursor_theme (GdkWaylandDisplay *display_wayland,
     return get_cursor_theme (display_wayland, "default", size);
 
   /* This may fall back to builtin cursors */
-  return wl_cursor_theme_create ("/usr/share/icons/default/cursors", size, display_wayland->shm);
+  return wl_cursor_theme_create ("/usr/share/icons/Adwaita/cursors", size, display_wayland->shm);
 }
 
 /**
@@ -1293,7 +1295,6 @@ typedef struct _GdkWaylandCairoSurfaceData {
   struct wl_shm_pool *pool;
   struct wl_buffer *buffer;
   GdkWaylandDisplay *display;
-  GdkFractionalScale scale;
 } GdkWaylandCairoSurfaceData;
 
 static int
@@ -1302,20 +1303,20 @@ open_shared_memory (void)
   static gboolean force_shm_open = FALSE;
   int ret = -1;
 
-#if !defined (__NR_memfd_create)
+#if !defined (HAVE_MEMFD_CREATE)
   force_shm_open = TRUE;
 #endif
 
   do
     {
-#if defined (__NR_memfd_create)
+#if defined (HAVE_MEMFD_CREATE)
       if (!force_shm_open)
         {
           int options = MFD_CLOEXEC;
 #if defined (MFD_ALLOW_SEALING)
           options |= MFD_ALLOW_SEALING;
 #endif
-          ret = syscall (__NR_memfd_create, "gdk-wayland", options);
+          ret = memfd_create ("gdk-wayland", options);
 
           /* fall back to shm_open until debian stops shipping 3.16 kernel
            * See bug 766341
@@ -1418,28 +1419,23 @@ gdk_wayland_cairo_surface_destroy (void *p)
 }
 
 cairo_surface_t *
-gdk_wayland_display_create_shm_surface (GdkWaylandDisplay        *display,
-                                        int                       width,
-                                        int                       height,
-                                        const GdkFractionalScale *scale)
+gdk_wayland_display_create_shm_surface (GdkWaylandDisplay *display,
+                                        guint              width,
+                                        guint              height)
 {
   GdkWaylandCairoSurfaceData *data;
   cairo_surface_t *surface = NULL;
   cairo_status_t status;
-  int scaled_width, scaled_height;
   int stride;
 
   data = g_new (GdkWaylandCairoSurfaceData, 1);
   data->display = display;
   data->buffer = NULL;
-  data->scale = *scale;
 
-  scaled_width = gdk_fractional_scale_scale (scale, width);
-  scaled_height = gdk_fractional_scale_scale (scale, height);
-  stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32, scaled_width);
+  stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32, width);
 
   data->pool = create_shm_pool (display->shm,
-                                scaled_height * stride,
+                                height * stride,
                                 &data->buf_length,
                                 &data->buf);
   if (G_UNLIKELY (data->pool == NULL))
@@ -1447,21 +1443,17 @@ gdk_wayland_display_create_shm_surface (GdkWaylandDisplay        *display,
 
   surface = cairo_image_surface_create_for_data (data->buf,
                                                  CAIRO_FORMAT_ARGB32,
-                                                 scaled_width,
-                                                 scaled_height,
+                                                 width,
+                                                 height,
                                                  stride);
 
   data->buffer = wl_shm_pool_create_buffer (data->pool, 0,
-                                            scaled_width, scaled_height,
+                                            width, height,
                                             stride, WL_SHM_FORMAT_ARGB8888);
 
   cairo_surface_set_user_data (surface, &gdk_wayland_shm_surface_cairo_key,
                                data, gdk_wayland_cairo_surface_destroy);
 
-  cairo_surface_set_device_scale (surface,
-                                  gdk_fractional_scale_to_double (scale),
-                                  gdk_fractional_scale_to_double (scale));
-
   status = cairo_surface_status (surface);
   if (status != CAIRO_STATUS_SUCCESS)
     {
@@ -1965,7 +1957,7 @@ init_settings (GdkDisplay *display)
   GSettings *settings;
   int i;
 
-  if (gdk_should_use_portal () &&
+  if (gdk_display_should_use_portal (display, PORTAL_SETTINGS_INTERFACE, 0) &&
       !(gdk_display_get_debug_flags (display) & GDK_DEBUG_DEFAULT_SETTINGS))
     {
       GVariant *ret;
@@ -2032,7 +2024,7 @@ init_settings (GdkDisplay *display)
               if (entry)
                 {
                   char *a = g_variant_print (v, FALSE);
-                  g_debug ("Using portal setting for %s %s: %s\n", schema_str, key, a);
+                  g_debug ("Using portal setting for %s %s: %s", schema_str, key, a);
                   g_free (a);
                   entry->valid = TRUE;
                   apply_portal_setting (entry, v, display);
@@ -2360,25 +2352,6 @@ subpixel_to_string (int layout)
   return NULL;
 }
 
-static void
-update_scale (GdkDisplay *display)
-{
-  GList *seats;
-  GList *l;
-
-  g_list_foreach (gdk_wayland_display_get_toplevel_surfaces (display),
-                  (GFunc)gdk_wayland_surface_update_scale,
-                  NULL);
-  seats = gdk_display_list_seats (display);
-  for (l = seats; l; l = l->next)
-    {
-      GdkSeat *seat = l->data;
-
-      gdk_wayland_seat_update_cursor_scale (GDK_WAYLAND_SEAT (seat));
-    }
-  g_list_free (seats);
-}
-
 static void
 gdk_wayland_display_init_xdg_output (GdkWaylandDisplay *self)
 {
@@ -2475,8 +2448,6 @@ apply_monitor_change (GdkWaylandMonitor *monitor)
 
   monitor->wl_output_done = FALSE;
   monitor->xdg_output_done = FALSE;
-
-  update_scale (GDK_MONITOR (monitor)->display);
 }
 
 static void
@@ -2751,7 +2722,6 @@ gdk_wayland_display_remove_output (GdkWaylandDisplay *self,
         {
           g_list_store_remove (self->monitors, i);
           gdk_monitor_invalidate (GDK_MONITOR (monitor));
-          update_scale (GDK_DISPLAY (self));
           g_object_unref (monitor);
           break;
         }
@@ -2773,19 +2743,6 @@ gdk_wayland_display_get_output_refresh_rate (GdkWaylandDisplay *display_wayland,
   return 0;
 }
 
-guint32
-gdk_wayland_display_get_output_scale (GdkWaylandDisplay *display_wayland,
-				      struct wl_output  *output)
-{
-  GdkWaylandMonitor *monitor;
-
-  monitor = get_monitor_for_output (display_wayland, output);
-  if (monitor != NULL)
-    return gdk_monitor_get_scale_factor (GDK_MONITOR (monitor));
-
-  return 0;
-}
-
 /**
  * gdk_wayland_display_query_registry:
  * @display: (type GdkWaylandDisplay): a `GdkDisplay`
