@@ -40,13 +40,6 @@ gdk_wayland_vulkan_context_create_surface (GdkVulkanContext *context,
   GdkSurface *surface = gdk_draw_context_get_surface (GDK_DRAW_CONTEXT (context));
   GdkDisplay *display = gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context));
 
-  /* This is necessary so that Vulkan sees the Surface.
-   * Usually, vkCreateXlibSurfaceKHR() will not cause a problem to happen as
-   * it just creates resources, but further calls with the resulting surface
-   * do cause issues.
-   */
-  gdk_display_sync (display);
-
   return GDK_VK_CHECK (vkCreateWaylandSurfaceKHR, gdk_vulkan_context_get_instance (context),
                                                    &(VkWaylandSurfaceCreateInfoKHR) {
                                                        VK_STRUCTURE_TYPE_WAYLAND_SURFACE_CREATE_INFO_KHR,
@@ -94,6 +87,7 @@ gdk_wayland_vulkan_context_class_init (GdkWaylandVulkanContextClass *klass)
   GdkDrawContextClass *draw_context_class = GDK_DRAW_CONTEXT_CLASS (klass);
 
   vulkan_context_class->create_surface = gdk_wayland_vulkan_context_create_surface;
+
   draw_context_class->end_frame = gdk_vulkan_context_wayland_end_frame;
   draw_context_class->empty_frame = gdk_vulkan_context_wayland_empty_frame;
 }
