@@ -2,7 +2,7 @@
 
 #include "gskgpushaderopprivate.h"
 
-#include "gskrendernode.h"
+#include "gskrendernodeprivate.h"
 
 #include <graphene.h>
 
@@ -10,15 +10,23 @@ G_BEGIN_DECLS
 
 void                    gsk_gpu_linear_gradient_op                      (GskGpuFrame                    *frame,
                                                                          GskGpuShaderClip                clip,
-                                                                         GskGpuColorStates               color_states,
+                                                                         GdkColorState                  *ccs,
+                                                                         float                           opacity,
+                                                                         const graphene_point_t         *offset,
+                                                                         GdkColorState                  *ics,
+                                                                         GskHueInterpolation             hue_interp,
                                                                          gboolean                        repeating,
                                                                          const graphene_rect_t          *rect,
                                                                          const graphene_point_t         *start,
                                                                          const graphene_point_t         *end,
-                                                                         const graphene_point_t         *offset,
-                                                                         const GskColorStop             *stops,
+                                                                         const GskColorStop2            *stops,
                                                                          gsize                           n_stops);
 
 
+void gsk_adjust_hue (GdkColorState       *ics,
+                     GskHueInterpolation  interp,
+                     const float          color1[4],
+                     float                color2[4]);
+
 G_END_DECLS
 
