@@ -333,8 +333,11 @@ gtk_application_impl_dbus_startup (GtkApplicationImpl *impl,
   dbus->object_path = g_application_get_dbus_object_path (G_APPLICATION (impl->application));
   dbus->unique_name = g_dbus_connection_get_unique_name (dbus->session);
 
-  if (gdk_should_use_portal ())
-    goto out;
+  if (gdk_display_should_use_portal (impl->display, PORTAL_INHIBIT_INTERFACE, 0))
+    {
+      g_debug ("Not using session manager");
+      goto out;
+    }
 
   dbus->cancellable = g_cancellable_new ();
 
@@ -504,12 +507,16 @@ gtk_application_impl_dbus_startup (GtkApplicationImpl *impl,
     }
 
   if (!same_bus)
-    g_object_set (gtk_settings_get_default (),
-                  "gtk-shell-shows-app-menu", FALSE,
-                  "gtk-shell-shows-menubar", FALSE,
-                  NULL);
+    {
+      g_object_set (gtk_settings_get_default (),
+                    "gtk-shell-shows-app-menu", FALSE,
+                    "gtk-shell-shows-menubar", FALSE,
+                    NULL);
+    }
 
-  if (dbus->sm_proxy == NULL && dbus->session)
+  if (dbus->sm_proxy == NULL &&
+      dbus->session != NULL &&
+      gdk_display_should_use_portal (impl->display, PORTAL_INHIBIT_INTERFACE, 0))
     {
       dbus->inhibit_proxy = gtk_application_get_proxy_if_service_present (dbus->session,
                                                                           G_DBUS_PROXY_FLAGS_NONE,
@@ -755,6 +762,13 @@ gtk_application_impl_dbus_inhibit (GtkApplicationImpl         *impl,
     {
       GVariantBuilder options;
 
+      if (reason == NULL)
+        /* Translators: This is the 'reason' given when inhibiting
+         * suspend or screen locking, and the caller hasn't specified
+         * a reason.
+         */
+        reason = _("Reason not specified");
+
       g_variant_builder_init (&options, G_VARIANT_TYPE_VARDICT);
       g_variant_builder_add (&options, "{sv}", "reason", g_variant_new_string (reason));
       res = g_dbus_proxy_call_sync (dbus->inhibit_proxy,
