@@ -29,8 +29,12 @@
 #include <gdk/wayland/gdkwayland.h>
 #endif
 
-#ifdef GDK_WINDOWING_QUARTZ
-#include <gdk/quartz/gdkquartz.h>
+#ifdef GDK_WINDOWING_MACOS
+#include <gdk/macos/gdkmacos.h>
+#endif
+
+#ifdef GDK_WINDOWING_ANDROID
+#include <gdk/android/gdkandroid.h>
 #endif
 
 G_DEFINE_TYPE (GtkApplicationImpl, gtk_application_impl, G_TYPE_OBJECT)
@@ -172,11 +176,16 @@ gtk_application_impl_new (GtkApplication *application,
     impl_type = gtk_application_impl_wayland_get_type ();
 #endif
 
-#ifdef GDK_WINDOWING_QUARTZ
-  if (GDK_IS_QUARTZ_DISPLAY (display))
+#ifdef GDK_WINDOWING_MACOS
+  if (GDK_IS_MACOS_DISPLAY (display))
     impl_type = gtk_application_impl_quartz_get_type ();
 #endif
 
+#ifdef GDK_WINDOWING_ANDROID
+  if (GDK_IS_ANDROID_DISPLAY (display))
+    impl_type = gtk_application_impl_android_get_type ();
+#endif
+
   impl = g_object_new (impl_type, NULL);
   impl->application = application;
   impl->display = display;
