@@ -15,19 +15,17 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_CSS_PROVIDER_PRIVATE_H__
-#define __GTK_CSS_PROVIDER_PRIVATE_H__
+#pragma once
 
 #include "gtkcssprovider.h"
 
 G_BEGIN_DECLS
 
-gchar *_gtk_get_theme_dir (void);
+char *_gtk_get_theme_dir (void) G_GNUC_MALLOC;
 
-const gchar *_gtk_css_provider_get_theme_dir (GtkCssProvider *provider);
+const char *_gtk_css_provider_get_theme_dir (GtkCssProvider *provider);
 
 void   gtk_css_provider_set_keep_css_sections (void);
 
 G_END_DECLS
 
-#endif /* __GTK_CSS_PROVIDER_PRIVATE_H__ */
