@@ -22,10 +22,12 @@
 /**
  * GtkExpander:
  *
- * `GtkExpander` allows the user to reveal its child by clicking
- * on an expander triangle.
+ * Allows the user to reveal or conceal a child widget.
  *
- * ![An example GtkExpander](expander.png)
+ * <picture>
+ *   <source srcset="expander-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkExpander" src="expander.png">
+ * </picture>
  *
  * This is similar to the triangles used in a `GtkTreeView`.
  *
@@ -113,7 +115,7 @@
  *
  * # Accessibility
  *
- * `GtkExpander` uses the %GTK_ACCESSIBLE_ROLE_BUTTON role.
+ * `GtkExpander` uses the [enum@Gtk.AccessibleRole.button] role.
  */
 
 #include "config.h"
@@ -131,6 +133,7 @@
 #include "gtkmain.h"
 #include "gtkprivate.h"
 #include "gtkwidgetprivate.h"
+#include "gtkbuilderprivate.h"
 
 #include <string.h>
 
@@ -310,7 +313,7 @@ gtk_expander_class_init (GtkExpanderClass *klass)
   klass->activate = gtk_expander_activate;
 
   /**
-   * GtkExpander:expanded: (attributes org.gtk.Property.get=gtk_expander_get_expanded org.gtk.Property.set=gtk_expander_set_expanded)
+   * GtkExpander:expanded:
    *
    * Whether the expander has been opened to reveal the child.
    */
@@ -321,7 +324,7 @@ gtk_expander_class_init (GtkExpanderClass *klass)
                                                          GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkExpander:label: (attributes org.gtk.Property.get=gtk_expander_get_label org.gtk.Property.set=gtk_expander_set_label)
+   * GtkExpander:label:
    *
    * The text of the expanders label.
    */
@@ -332,7 +335,7 @@ gtk_expander_class_init (GtkExpanderClass *klass)
                                                         GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT));
 
   /**
-   * GtkExpander:use-underline: (attributes org.gtk.Property.get=gtk_expander_get_use_underline org.gtk.Property.set=gtk_expander_set_use_underline)
+   * GtkExpander:use-underline:
    *
    * Whether an underline in the text indicates a mnemonic.
    */
@@ -343,7 +346,7 @@ gtk_expander_class_init (GtkExpanderClass *klass)
                                                          GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkExpander:use-markup: (attributes org.gtk.Property.get=gtk_expander_get_use_markup org.gtk.Property.set=gtk_expander_set_use_markup)
+   * GtkExpander:use-markup:
    *
    * Whether the text in the label is Pango markup.
    */
@@ -354,7 +357,7 @@ gtk_expander_class_init (GtkExpanderClass *klass)
                                                          GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkExpander:label-widget: (attributes org.gtk.Property.get=gtk_expander_get_label_widget org.gtk.Property.set=gtk_expander_set_label_widget)
+   * GtkExpander:label-widget:
    *
    * A widget to display instead of the usual expander label.
    */
@@ -365,7 +368,7 @@ gtk_expander_class_init (GtkExpanderClass *klass)
                                                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkExpander:resize-toplevel: (attributes org.gtk.Property.get=gtk_expander_get_resize_toplevel org.gtk.Property.set=gtk_expander_set_resize_toplevel)
+   * GtkExpander:resize-toplevel:
    *
    * When this property is %TRUE, the expander will resize the toplevel
    * widget containing the expander upon expanding and collapsing.
@@ -377,7 +380,7 @@ gtk_expander_class_init (GtkExpanderClass *klass)
                                                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkExpander:child: (attributes org.gtk.Property.get=gtk_expander_get_child org.gtk.Property.set=gtk_expander_set_child)
+   * GtkExpander:child:
    *
    * The child widget.
    */
@@ -466,11 +469,19 @@ gtk_expander_buildable_add_child (GtkBuildable  *buildable,
                                   const char    *type)
 {
   if (g_strcmp0 (type, "label") == 0)
-    gtk_expander_set_label_widget (GTK_EXPANDER (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, "label", "label-widget");
+      gtk_expander_set_label_widget (GTK_EXPANDER (buildable), GTK_WIDGET (child));
+    }
   else if (GTK_IS_WIDGET (child))
-    gtk_expander_set_child (GTK_EXPANDER (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "child");
+      gtk_expander_set_child (GTK_EXPANDER (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -857,7 +868,7 @@ gtk_expander_new_with_mnemonic (const char *label)
 }
 
 /**
- * gtk_expander_set_expanded: (attributes org.gtk.Method.set_property=expanded)
+ * gtk_expander_set_expanded:
  * @expander: a `GtkExpander`
  * @expanded: whether the child widget is revealed
  *
@@ -919,7 +930,7 @@ gtk_expander_set_expanded (GtkExpander *expander,
 }
 
 /**
- * gtk_expander_get_expanded: (attributes org.gtk.Method.get_property=expanded)
+ * gtk_expander_get_expanded:
  * @expander:a `GtkExpander`
  *
  * Queries a `GtkExpander` and returns its current state.
@@ -937,7 +948,7 @@ gtk_expander_get_expanded (GtkExpander *expander)
 }
 
 /**
- * gtk_expander_set_label: (attributes org.gtk.Method.set_property=label)
+ * gtk_expander_set_label:
  * @expander: a `GtkExpander`
  * @label: (nullable): a string
  *
@@ -970,7 +981,7 @@ gtk_expander_set_label (GtkExpander *expander,
 }
 
 /**
- * gtk_expander_get_label: (attributes org.gtk.Method.get_property=label)
+ * gtk_expander_get_label:
  * @expander: a `GtkExpander`
  *
  * Fetches the text from a label widget.
@@ -996,7 +1007,7 @@ gtk_expander_get_label (GtkExpander *expander)
 }
 
 /**
- * gtk_expander_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * gtk_expander_set_use_underline:
  * @expander: a `GtkExpander`
  * @use_underline: %TRUE if underlines in the text indicate mnemonics
  *
@@ -1022,7 +1033,7 @@ gtk_expander_set_use_underline (GtkExpander *expander,
 }
 
 /**
- * gtk_expander_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * gtk_expander_get_use_underline:
  * @expander: a `GtkExpander`
  *
  * Returns whether an underline in the text indicates a mnemonic.
@@ -1039,7 +1050,7 @@ gtk_expander_get_use_underline (GtkExpander *expander)
 }
 
 /**
- * gtk_expander_set_use_markup: (attributes org.gtk.Method.set_property=use-markup)
+ * gtk_expander_set_use_markup:
  * @expander: a `GtkExpander`
  * @use_markup: %TRUE if the label’s text should be parsed for markup
  *
@@ -1065,7 +1076,7 @@ gtk_expander_set_use_markup (GtkExpander *expander,
 }
 
 /**
- * gtk_expander_get_use_markup: (attributes org.gtk.Method.get_property=use-markup)
+ * gtk_expander_get_use_markup:
  * @expander: a `GtkExpander`
  *
  * Returns whether the label’s text is interpreted as Pango markup.
@@ -1081,7 +1092,7 @@ gtk_expander_get_use_markup (GtkExpander *expander)
 }
 
 /**
- * gtk_expander_set_label_widget: (attributes org.gtk.Method.set_property=label-widget)
+ * gtk_expander_set_label_widget:
  * @expander: a `GtkExpander`
  * @label_widget: (nullable): the new label widget
  *
@@ -1125,7 +1136,7 @@ gtk_expander_set_label_widget (GtkExpander *expander,
 }
 
 /**
- * gtk_expander_get_label_widget: (attributes org.gtk.Method.get_property=label-widget)
+ * gtk_expander_get_label_widget:
  * @expander: a `GtkExpander`
  *
  * Retrieves the label widget for the frame.
@@ -1141,7 +1152,7 @@ gtk_expander_get_label_widget (GtkExpander *expander)
 }
 
 /**
- * gtk_expander_set_resize_toplevel: (attributes org.gtk.Method.set_property=resize-toplevel)
+ * gtk_expander_set_resize_toplevel:
  * @expander: a `GtkExpander`
  * @resize_toplevel: whether to resize the toplevel
  *
@@ -1162,7 +1173,7 @@ gtk_expander_set_resize_toplevel (GtkExpander *expander,
 }
 
 /**
- * gtk_expander_get_resize_toplevel: (attributes org.gtk.Method.get_property=resize-toplevel)
+ * gtk_expander_get_resize_toplevel:
  * @expander: a `GtkExpander`
  *
  * Returns whether the expander will resize the toplevel widget
@@ -1179,7 +1190,7 @@ gtk_expander_get_resize_toplevel (GtkExpander *expander)
 }
 
 /**
- * gtk_expander_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_expander_set_child:
  * @expander: a `GtkExpander`
  * @child: (nullable): the child widget
  *
@@ -1237,7 +1248,7 @@ gtk_expander_set_child (GtkExpander *expander,
 }
 
 /**
- * gtk_expander_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_expander_get_child:
  * @expander: a `GtkExpander`
  *
  * Gets the child widget of @expander.
