@@ -22,6 +22,7 @@
 #include "gtkfilechoosernativeprivate.h"
 #include "gtknativedialogprivate.h"
 
+#include <glib/gi18n-lib.h>
 #include "gtkprivate.h"
 #include "deprecated/gtkdialog.h"
 #include "gtkfilechooserprivate.h"
@@ -34,6 +35,7 @@
 #include "gtkmain.h"
 #include "gtkfilefilterprivate.h"
 #include "gtkwindowprivate.h"
+#include "gtkalertdialog.h"
 
 
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
@@ -54,7 +56,6 @@ typedef struct {
 
   char *exported_handle;
   GtkWindow *exported_window;
-  PortalErrorHandler error_handler;
 } FilechooserPortalData;
 
 
@@ -234,9 +235,9 @@ send_close (FilechooserPortalData *data)
 }
 
 static void
-open_file_msg_cb (GObject *source_object,
+open_file_msg_cb (GObject      *source_object,
                   GAsyncResult *res,
-                  gpointer user_data)
+                  gpointer      user_data)
 {
   FilechooserPortalData *data = user_data;
   GtkFileChooserNative *self = data->self;
@@ -251,12 +252,13 @@ open_file_msg_cb (GObject *source_object,
 
   if (reply == NULL)
     {
-      if (!data->hidden && data->error_handler)
+      if (!data->hidden)
         {
-          data->error_handler (self);
           filechooser_portal_data_free (data);
           self->mode_data = NULL;
         }
+
+      /* FIXME: Show an error dialog here ? */
       g_error_free (error);
       return;
     }
@@ -305,7 +307,7 @@ get_filters (GtkFileChooser *self)
   for (i = 0; i < n; i++)
     {
       GtkFileFilter *filter = g_list_model_get_item (filters, i);
-      g_variant_builder_add (&builder, "@(sa(us))", gtk_file_filter_to_gvariant (filter));
+      g_variant_builder_add_value (&builder, gtk_file_filter_to_gvariant (filter));
       g_object_unref (filter);
     }
   g_object_unref (filters);
@@ -344,8 +346,7 @@ serialize_choices (GtkFileChooserNative *self)
     {
       GtkFileChooserNativeChoice *choice = l->data;
 
-      g_variant_builder_add (&builder, "@(ssa(ss)s)",
-                             gtk_file_chooser_native_choice_to_variant (choice));
+      g_variant_builder_add_value (&builder, gtk_file_chooser_native_choice_to_variant (choice));
     }
 
   return g_variant_builder_end (&builder);
@@ -418,12 +419,11 @@ show_portal_file_chooser (GtkFileChooserNative *self,
     }
   if (self->current_file)
     {
-      char *path;
+      const char *path;
 
-      path = g_file_get_path (GTK_FILE_CHOOSER_NATIVE (self)->current_file);
+      path = g_file_peek_path (GTK_FILE_CHOOSER_NATIVE (self)->current_file);
       g_variant_builder_add (&opt_builder, "{sv}", "current_file",
                              g_variant_new_bytestring (path));
-      g_free (path);
     }
 
   if (self->choices)
@@ -469,47 +469,58 @@ window_handle_exported (GtkWindow  *window,
 }
 
 gboolean
-gtk_file_chooser_native_portal_show (GtkFileChooserNative *self,
-                                     PortalErrorHandler    error_handler)
+gtk_file_chooser_native_portal_show (GtkFileChooserNative *self)
 {
   FilechooserPortalData *data;
   GtkWindow *transient_for;
   GDBusConnection *connection;
-  GtkFileChooserAction action;
   const char *method_name;
+  GdkDisplay *display;
 
-  if (!self->use_portal && !gdk_should_use_portal ())
+  transient_for = gtk_native_dialog_get_transient_for (GTK_NATIVE_DIALOG (self));
+
+  if (transient_for)
+    display = gtk_widget_get_display (GTK_WIDGET (transient_for));
+  else
+    display = gdk_display_get_default ();
+
+  if (!gdk_display_should_use_portal (display, PORTAL_FILECHOOSER_INTERFACE, 3))
     return FALSE;
 
+  /* From here on out, we want to return TRUE, since we should use the portal,
+   * or fail.
+   */
+
   connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
   if (connection == NULL)
-    return FALSE;
-
-  action = gtk_file_chooser_get_action (GTK_FILE_CHOOSER (self));
-
-  if (action == GTK_FILE_CHOOSER_ACTION_OPEN)
-    method_name = "OpenFile";
-  else if (action == GTK_FILE_CHOOSER_ACTION_SAVE)
-    method_name = "SaveFile";
-  else if (action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
     {
-      if (gtk_get_portal_interface_version (connection, "org.freedesktop.portal.FileChooser") < 3)
-        {
-          g_warning ("GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER is not supported by GtkFileChooserNativePortal because portal is too old");
-          return FALSE;
-        }
-      method_name = "OpenFile";
+      GtkAlertDialog *alert;
+
+      alert = gtk_alert_dialog_new (_("The session bus is not available"));
+      gtk_alert_dialog_show (alert, transient_for);
+      g_object_unref (alert);
+
+      return TRUE;
     }
-  else
+
+  switch (gtk_file_chooser_get_action (GTK_FILE_CHOOSER (self)))
     {
-      g_warning ("GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER is not supported by GtkFileChooserNativePortal");
-      return FALSE;
+    case GTK_FILE_CHOOSER_ACTION_OPEN:
+      method_name = "OpenFile";
+      break;
+    case GTK_FILE_CHOOSER_ACTION_SAVE:
+      method_name = "SaveFile";
+      break;
+    case GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER:
+      method_name = "OpenFile";
+      break;
+    default:
+      g_assert_not_reached ();
     }
 
   data = g_new0 (FilechooserPortalData, 1);
   data->self = g_object_ref (self);
   data->connection = connection;
-  data->error_handler = error_handler;
 
   data->method_name = method_name;
 
@@ -518,7 +529,6 @@ gtk_file_chooser_native_portal_show (GtkFileChooserNative *self,
 
   self->mode_data = data;
 
-  transient_for = gtk_native_dialog_get_transient_for (GTK_NATIVE_DIALOG (self));
   if (transient_for != NULL && gtk_widget_is_visible (GTK_WIDGET (transient_for)))
     {
       if (!gtk_window_export_handle (transient_for,
@@ -546,8 +556,8 @@ gtk_file_chooser_native_portal_hide (GtkFileChooserNative *self)
 {
   FilechooserPortalData *data = self->mode_data;
 
-  /* This is always set while dialog visible */
-  g_assert (data != NULL);
+  if (data == NULL)
+    return;
 
   data->hidden = TRUE;
 
