@@ -24,32 +24,32 @@
 /*< private >
  * GtkMenuTracker:
  *
- * #GtkMenuTracker is a simple object to ease implementations of #GMenuModel.
- * Given a #GtkActionObservable (usually a #GActionMuxer) along with a
- * #GMenuModel, it will tell you which menu items to create and where to place
+ * GtkMenuTracker is a simple object to ease implementations of GMenuModel.
+ * Given a GtkActionObservable (usually a GActionMuxer) along with a
+ * GMenuModel, it will tell you which menu items to create and where to place
  * them. If a menu item is removed, it will tell you the position of the menu
  * item to remove.
  *
- * Using #GtkMenuTracker is fairly simple. The only guarantee you must make
- * to #GtkMenuTracker is that you must obey all insert signals and track the
- * position of items that #GtkMenuTracker gives you. That is, #GtkMenuTracker
+ * Using GtkMenuTracker is fairly simple. The only guarantee you must make
+ * to GtkMenuTracker is that you must obey all insert signals and track the
+ * position of items that GtkMenuTracker gives you. That is, GtkMenuTracker
  * expects positions of all the latter items to change when it calls your
  * insertion callback with an early position, as it may ask you to remove
  * an item with a readjusted position later.
  *
- * #GtkMenuTracker will give you a #GtkMenuTrackerItem in your callback. You
+ * GtkMenuTracker will give you a GtkMenuTrackerItem in your callback. You
  * must hold onto this object until a remove signal is emitted. This item
  * represents a single menu item, which can be one of three classes: normal item,
  * separator, or submenu.
  *
- * Certain properties on the #GtkMenuTrackerItem are mutable, and you must
+ * Certain properties on the GtkMenuTrackerItem are mutable, and you must
  * listen for changes in the item. For more details, see the documentation
- * for #GtkMenuTrackerItem along with https://wiki.gnome.org/Projects/GLib/GApplication/GMenuModel.
+ * for GtkMenuTrackerItem along with GMenuModel's reference.
  *
  * The idea of @with_separators is for special cases where menu models may
  * be tracked in places where separators are not available, like in toplevel
- * "File", “Edit” menu bars. Ignoring separator items is wrong, as #GtkMenuTracker
- * expects the position to change, so we must tell #GtkMenuTracker to ignore
+ * "File", “Edit” menu bars. Ignoring separator items is wrong, as GtkMenuTracker
+ * expects the position to change, so we must tell GtkMenuTracker to ignore
  * separators itself.
  */
 
@@ -377,7 +377,7 @@ gtk_menu_tracker_add_items (GtkMenuTracker         *tracker,
             {
               GtkMenuTrackerSection *fake_section;
 
-              fake_section = g_slice_new0 (GtkMenuTrackerSection);
+              fake_section = g_new0 (GtkMenuTrackerSection, 1);
               fake_section->is_fake = TRUE;
               fake_section->model = g_object_ref (item);
               fake_section->handler = g_signal_connect (item, "notify::is-visible",
@@ -467,7 +467,7 @@ gtk_menu_tracker_section_free (GtkMenuTrackerSection *section)
   g_slist_free_full (section->items, (GDestroyNotify) gtk_menu_tracker_section_free);
   g_free (section->action_namespace);
   g_object_unref (section->model);
-  g_slice_free (GtkMenuTrackerSection, section);
+  g_free (section);
 }
 
 static GtkMenuTrackerSection *
@@ -480,7 +480,7 @@ gtk_menu_tracker_section_new (GtkMenuTracker *tracker,
 {
   GtkMenuTrackerSection *section;
 
-  section = g_slice_new0 (GtkMenuTrackerSection);
+  section = g_new0 (GtkMenuTrackerSection, 1);
   section->model = g_object_ref (model);
   section->with_separators = with_separators;
   section->action_namespace = g_strdup (action_namespace);
@@ -540,10 +540,10 @@ gtk_menu_tracker_section_new (GtkMenuTracker *tracker,
  * this case -- to the section menu item corresponding to the separator.
  * This is useful if the section specifies a label, for example.  If
  * there is an “action-namespace” attribute on this menu item then it
- * should be ignored by the consumer because #GtkMenuTracker has already
+ * should be ignored by the consumer because GtkMenuTracker has already
  * handled it.
  *
- * When using #GtkMenuTracker there is no need to hold onto @model or
+ * When using GtkMenuTracker there is no need to hold onto @model or
  * monitor it for changes.  The model will be unreffed when
  * gtk_menu_tracker_free() is called.
  */
@@ -560,7 +560,7 @@ gtk_menu_tracker_new (GtkActionObservable      *observable,
 {
   GtkMenuTracker *tracker;
 
-  tracker = g_slice_new (GtkMenuTracker);
+  tracker = g_new (GtkMenuTracker, 1);
   tracker->merge_sections = merge_sections;
   tracker->mac_os_mode = mac_os_mode;
   tracker->observable = g_object_ref (observable);
@@ -602,7 +602,7 @@ gtk_menu_tracker_new_for_item_link (GtkMenuTrackerItem       *item,
 
 /*< private >
  * gtk_menu_tracker_free:
- * @tracker: a #GtkMenuTracker
+ * @tracker: a GtkMenuTracker
  *
  * Frees the tracker, ...
  */
@@ -611,5 +611,5 @@ gtk_menu_tracker_free (GtkMenuTracker *tracker)
 {
   gtk_menu_tracker_section_free (tracker->toplevel);
   g_object_unref (tracker->observable);
-  g_slice_free (GtkMenuTracker, tracker);
+  g_free (tracker);
 }
