@@ -118,23 +118,23 @@
  *
  * # CSS nodes
  *
- * |[<!-- language="plain" -->
+ * ```
  * modelbutton
  * ├── <child>
  * ╰── check
- * ]|
+ * ```
  *
- * |[<!-- language="plain" -->
+ * ```
  * modelbutton
  * ├── <child>
  * ╰── radio
- * ]|
+ * ```
  *
- * |[<!-- language="plain" -->
+ * ```
  * modelbutton
  * ├── <child>
  * ╰── arrow
- * ]|
+ * ```
  *
  * GtkModelButton has a main CSS node with name modelbutton, and a subnode,
  * which will have the name check, radio or arrow, depending on the role
@@ -143,11 +143,11 @@
  * The subnode is positioned before or after the content nodes and gets the
  * .left or .right style class, depending on where it is located.
  *
- * |[<!-- language="plain" -->
+ * ```
  * button.model
  * ├── <child>
  * ╰── check
- * ]|
+ * ```
  *
  * Iconic model buttons (see GtkModelButton:iconic) change the name of
  * their main node to button and add a .model style class to it. The indicator
@@ -1067,10 +1067,29 @@ switch_menu (GtkModelButton *button)
   stack = gtk_widget_get_ancestor (GTK_WIDGET (button), GTK_TYPE_STACK);
   if (stack != NULL)
     {
+      if (button->role == GTK_BUTTON_ROLE_NORMAL)
+        {
+          GtkWidget *title_button = gtk_widget_get_first_child (gtk_stack_get_child_by_name (GTK_STACK (stack), button->menu_name));
+          gtk_accessible_update_state (GTK_ACCESSIBLE (button),
+                                       GTK_ACCESSIBLE_STATE_EXPANDED, TRUE,
+                                       -1);
+          gtk_accessible_update_state (GTK_ACCESSIBLE (title_button),
+                                       GTK_ACCESSIBLE_STATE_EXPANDED, TRUE,
+                                       -1);
+          g_object_set_data (G_OBJECT (title_button), "-gtk-model-button-parent", button);
+        }
+      else if (button->role == GTK_BUTTON_ROLE_TITLE)
+        {
+          GtkWidget *parent_button = g_object_get_data (G_OBJECT (button), "-gtk-model-button-parent");
+          gtk_accessible_update_state (GTK_ACCESSIBLE (parent_button),
+                                        GTK_ACCESSIBLE_STATE_EXPANDED, FALSE,
+                                        -1);
+          gtk_accessible_update_state (GTK_ACCESSIBLE (button),
+                                        GTK_ACCESSIBLE_STATE_EXPANDED, FALSE,
+                                        -1);
+          g_object_set_data (G_OBJECT (button), "-gtk-model-button-parent", NULL);
+        }
       gtk_stack_set_visible_child_name (GTK_STACK (stack), button->menu_name);
-      gtk_accessible_update_state (GTK_ACCESSIBLE (button),
-                                   GTK_ACCESSIBLE_STATE_EXPANDED, TRUE,
-                                   -1);
     }
 }
 
@@ -1383,6 +1402,10 @@ open_submenu (gpointer data)
           gtk_popover_menu_set_open_submenu (GTK_POPOVER_MENU (popover), submenu);
           gtk_popover_menu_set_parent_menu (GTK_POPOVER_MENU (submenu), GTK_WIDGET (popover));
         }
+      else
+        {
+          gtk_popover_menu_close_submenus (GTK_POPOVER_MENU (popover));
+        }
     }
 
   button->open_timeout = 0;
