@@ -33,9 +33,12 @@
 /**
  * GtkPicture:
  *
- * The `GtkPicture` widget displays a `GdkPaintable`.
+ * Displays a `GdkPaintable`.
  *
- * ![An example GtkPicture](picture.png)
+ * picture>
+ *   <source srcset="picture-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkPicture" src="picture.png">
+ * </picture>
  *
  * Many convenience functions are provided to make pictures simple to use.
  * For example, if you want to load an image from a file, and then display
@@ -78,7 +81,7 @@
  *
  * ## Accessibility
  *
- * `GtkPicture` uses the `GTK_ACCESSIBLE_ROLE_IMG` role.
+ * `GtkPicture` uses the [enum@Gtk.AccessibleRole.img] role.
  */
 
 enum
@@ -229,6 +232,19 @@ gtk_picture_measure (GtkWidget      *widget,
                                            &min_width, &min_height);
     }
 
+  if (for_size > 0 && self->content_fit == GTK_CONTENT_FIT_SCALE_DOWN)
+    {
+      double opposite_intrinsic_size;
+
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
+        opposite_intrinsic_size = gdk_paintable_get_intrinsic_height (self->paintable);
+      else
+        opposite_intrinsic_size = gdk_paintable_get_intrinsic_width (self->paintable);
+
+      if (opposite_intrinsic_size != 0 && opposite_intrinsic_size < for_size)
+        for_size = opposite_intrinsic_size;
+    }
+
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       gdk_paintable_compute_concrete_size (self->paintable,
@@ -400,7 +416,7 @@ gtk_picture_class_init (GtkPictureClass *class)
   widget_class->measure = gtk_picture_measure;
 
   /**
-   * GtkPicture:paintable: (attributes org.gtk.Property.get=gtk_picture_get_paintable org.gtk.Property.set=gtk_picture_set_paintable)
+   * GtkPicture:paintable:
    *
    * The `GdkPaintable` to be displayed by this `GtkPicture`.
    */
@@ -410,7 +426,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPicture:file: (attributes org.gtk.Property.get=gtk_picture_get_file org.gtk.Property.set=gtk_picture_set_file)
+   * GtkPicture:file:
    *
    * The `GFile` that is displayed or %NULL if none.
    */
@@ -420,7 +436,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPicture:alternative-text: (attributes org.gtk.Property.get=gtk_picture_get_alternative_text org.gtk.Property.set=gtk_picture_set_alternative_text)
+   * GtkPicture:alternative-text:
    *
    * The alternative textual description for the picture.
    */
@@ -430,7 +446,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPicture:keep-aspect-ratio: (attributes org.gtk.Property.get=gtk_picture_get_keep_aspect_ratio org.gtk.Property.set=gtk_picture_set_keep_aspect_ratio)
+   * GtkPicture:keep-aspect-ratio:
    *
    * Whether the GtkPicture will render its contents trying to preserve the aspect
    * ratio.
@@ -445,7 +461,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                             G_PARAM_DEPRECATED);
 
   /**
-   * GtkPicture:can-shrink: (attributes org.gtk.Property.get=gtk_picture_get_can_shrink org.gtk.Property.set=gtk_picture_set_can_shrink)
+   * GtkPicture:can-shrink:
    *
    * If the `GtkPicture` can be made smaller than the natural size of its contents.
    */
@@ -455,7 +471,7 @@ gtk_picture_class_init (GtkPictureClass *class)
                             GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPicture:content-fit: (attributes org.gtk.Property.get=gtk_picture_get_content_fit org.gtk.Property.set=gtk_picture_set_content_fit)
+   * GtkPicture:content-fit:
    *
    * How the content should be resized to fit inside the `GtkPicture`.
    *
@@ -651,7 +667,7 @@ gtk_picture_new_for_resource (const char *resource_path)
 }
 
 /**
- * gtk_picture_set_file: (attributes org.gtk.Method.set_property=file)
+ * gtk_picture_set_file:
  * @self: a `GtkPicture`
  * @file: (nullable): a `GFile`
  *
@@ -688,7 +704,7 @@ gtk_picture_set_file (GtkPicture *self,
 }
 
 /**
- * gtk_picture_get_file: (attributes org.gtk.Method.get_property=file)
+ * gtk_picture_get_file:
  * @self: a `GtkPicture`
  *
  * Gets the `GFile` currently displayed if @self is displaying a file.
@@ -808,8 +824,22 @@ gtk_picture_set_pixbuf (GtkPicture *self,
     g_object_unref (texture);
 }
 
+static gboolean
+paintable_size_equal (GdkPaintable *one,
+                      GdkPaintable *two)
+{
+  if (one == NULL)
+    return two == NULL;
+  else if (two == NULL)
+    return FALSE;
+
+  return gdk_paintable_get_intrinsic_width (one) == gdk_paintable_get_intrinsic_width (two) &&
+         gdk_paintable_get_intrinsic_height (one) == gdk_paintable_get_intrinsic_height (two) &&
+         gdk_paintable_get_intrinsic_aspect_ratio (one) == gdk_paintable_get_intrinsic_aspect_ratio (two);
+}
+
 /**
- * gtk_picture_set_paintable: (attributes org.gtk.Method.set_property=paintable)
+ * gtk_picture_set_paintable:
  * @self: a `GtkPicture`
  * @paintable: (nullable): a `GdkPaintable`
  *
@@ -823,6 +853,8 @@ void
 gtk_picture_set_paintable (GtkPicture   *self,
                            GdkPaintable *paintable)
 {
+  gboolean size_changed;
+
   g_return_if_fail (GTK_IS_PICTURE (self));
   g_return_if_fail (paintable == NULL || GDK_IS_PAINTABLE (paintable));
 
@@ -834,6 +866,8 @@ gtk_picture_set_paintable (GtkPicture   *self,
   if (paintable)
     g_object_ref (paintable);
 
+  size_changed = !paintable_size_equal (self->paintable, paintable);
+ 
   gtk_picture_clear_paintable (self);
 
   self->paintable = paintable;
@@ -855,7 +889,10 @@ gtk_picture_set_paintable (GtkPicture   *self,
                           self);
     }
 
-  gtk_widget_queue_resize (GTK_WIDGET (self));
+  if (size_changed)
+    gtk_widget_queue_resize (GTK_WIDGET (self));
+  else
+    gtk_widget_queue_draw (GTK_WIDGET (self));
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PAINTABLE]);
 
@@ -863,7 +900,7 @@ gtk_picture_set_paintable (GtkPicture   *self,
 }
 
 /**
- * gtk_picture_get_paintable: (attributes org.gtk.Method.get_property=paintable)
+ * gtk_picture_get_paintable:
  * @self: a `GtkPicture`
  *
  * Gets the `GdkPaintable` being displayed by the `GtkPicture`.
@@ -879,7 +916,7 @@ gtk_picture_get_paintable (GtkPicture *self)
 }
 
 /**
- * gtk_picture_set_keep_aspect_ratio: (attributes org.gtk.Method.set_property=keep-aspect-ratio)
+ * gtk_picture_set_keep_aspect_ratio:
  * @self: a `GtkPicture`
  * @keep_aspect_ratio: whether to keep aspect ratio
  *
@@ -908,7 +945,7 @@ gtk_picture_set_keep_aspect_ratio (GtkPicture *self,
 }
 
 /**
- * gtk_picture_get_keep_aspect_ratio: (attributes org.gtk.Method.get_property=keep-aspect-ratio)
+ * gtk_picture_get_keep_aspect_ratio:
  * @self: a `GtkPicture`
  *
  * Returns whether the `GtkPicture` preserves its contents aspect ratio.
@@ -928,7 +965,7 @@ gtk_picture_get_keep_aspect_ratio (GtkPicture *self)
 }
 
 /**
- * gtk_picture_set_can_shrink: (attributes org.gtk.Method.set_property=can-shrink)
+ * gtk_picture_set_can_shrink:
  * @self: a `GtkPicture`
  * @can_shrink: if @self can be made smaller than its contents
  *
@@ -960,7 +997,7 @@ gtk_picture_set_can_shrink (GtkPicture *self,
 }
 
 /**
- * gtk_picture_get_can_shrink: (attributes org.gtk.Method.get_property=can-shrink)
+ * gtk_picture_get_can_shrink:
  * @self: a `GtkPicture`
  *
  * Returns whether the `GtkPicture` respects its contents size.
@@ -976,7 +1013,7 @@ gtk_picture_get_can_shrink (GtkPicture *self)
 }
 
 /**
- * gtk_picture_set_content_fit: (attributes org.gtk.Method.set_property=content-fit)
+ * gtk_picture_set_content_fit:
  * @self: a `GtkPicture`
  * @content_fit: the content fit mode
  *
@@ -991,6 +1028,7 @@ gtk_picture_set_content_fit (GtkPicture    *self,
                              GtkContentFit  content_fit)
 {
   gboolean notify_keep_aspect_ratio;
+  gboolean queue_resize;
 
   g_return_if_fail (GTK_IS_PICTURE (self));
 
@@ -999,10 +1037,15 @@ gtk_picture_set_content_fit (GtkPicture    *self,
 
   notify_keep_aspect_ratio = (content_fit == GTK_CONTENT_FIT_FILL ||
                               self->content_fit == GTK_CONTENT_FIT_FILL);
+  queue_resize = (content_fit == GTK_CONTENT_FIT_SCALE_DOWN ||
+                  self->content_fit == GTK_CONTENT_FIT_SCALE_DOWN);
 
   self->content_fit = content_fit;
 
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  if (queue_resize)
+    gtk_widget_queue_resize (GTK_WIDGET (self));
+  else
+    gtk_widget_queue_draw (GTK_WIDGET (self));
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CONTENT_FIT]);
 
@@ -1011,7 +1054,7 @@ gtk_picture_set_content_fit (GtkPicture    *self,
 }
 
 /**
- * gtk_picture_get_content_fit: (attributes org.gtk.Method.get_property=content-fit)
+ * gtk_picture_get_content_fit:
  * @self: a `GtkPicture`
  *
  * Returns the fit mode for the content of the `GtkPicture`.
@@ -1031,7 +1074,7 @@ gtk_picture_get_content_fit (GtkPicture *self)
 }
 
 /**
- * gtk_picture_set_alternative_text: (attributes org.gtk.Method.set_property=alternative-text)
+ * gtk_picture_set_alternative_text:
  * @self: a `GtkPicture`
  * @alternative_text: (nullable): a textual description of the contents
  *
@@ -1063,7 +1106,7 @@ gtk_picture_set_alternative_text (GtkPicture *self,
 }
 
 /**
- * gtk_picture_get_alternative_text: (attributes org.gtk.Method.get_property=alternative-text)
+ * gtk_picture_get_alternative_text:
  * @self: a `GtkPicture`
  *
  * Gets the alternative textual description of the picture.
