@@ -26,13 +26,16 @@
 
 #include "gtkprogressbar.h"
 
+#include "gtkaccessiblerange.h"
 #include "gtkboxlayout.h"
+#include "gtkcssboxesprivate.h"
 #include "gtkgizmoprivate.h"
-#include "gtkintl.h"
+#include <glib/gi18n-lib.h>
 #include "gtklabel.h"
 #include "gtkorientable.h"
 #include "gtkprogresstrackerprivate.h"
 #include "gtkprivate.h"
+#include "gtksnapshot.h"
 #include "gtkwidgetprivate.h"
 
 #include <string.h>
@@ -40,13 +43,15 @@
 /**
  * GtkProgressBar:
  *
- * `GtkProgressBar` is typically used to display the progress of a long
- * running operation.
+ * Displays the progress of a long-running operation.
  *
- * It provides a visual clue that processing is underway. `GtkProgressBar`
- * can be used in two different modes: percentage mode and activity mode.
+ * `GtkProgressBar` provides a visual clue that processing is underway.
+ * It can be used in two different modes: percentage mode and activity mode.
  *
- * ![An example GtkProgressBar](progressbar.png)
+ * <picture>
+ *   <source srcset="progressbar-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkProgressBar" src="progressbar.png">
+ * </picture>
  *
  * When an application can determine how much work needs to take place
  * (e.g. read a fixed number of bytes from a file) and can monitor its
@@ -85,7 +90,7 @@
  *
  * # Accessibility
  *
- * `GtkProgressBar` uses the %GTK_ACCESSIBLE_ROLE_PROGRESS_BAR role.
+ * `GtkProgressBar` uses the [enum@Gtk.AccessibleRole.progress_bar] role.
  */
 
 typedef struct _GtkProgressBarClass         GtkProgressBarClass;
@@ -158,7 +163,8 @@ static void     gtk_progress_bar_direction_changed (GtkWidget        *widget,
                                                     GtkTextDirection  previous_dir);
 
 G_DEFINE_TYPE_WITH_CODE (GtkProgressBar, gtk_progress_bar, GTK_TYPE_WIDGET,
-                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL))
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE_RANGE, NULL))
 
 static void
 gtk_progress_bar_class_init (GtkProgressBarClass *class)
@@ -179,57 +185,49 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_override_property (gobject_class, PROP_ORIENTATION, "orientation");
 
   /**
-   * GtkProgressBar:inverted: (attributes org.gtk.Property.get=gtk_progress_bar_get_inverted org.gtk.Property.set=gtk_progress_bar_set_inverted)
+   * GtkProgressBar:inverted:
    *
    * Invert the direction in which the progress bar grows.
    */
   progress_props[PROP_INVERTED] =
-      g_param_spec_boolean ("inverted",
-                            P_("Inverted"),
-                            P_("Invert the direction in which the progress bar grows"),
+      g_param_spec_boolean ("inverted", NULL, NULL,
                             FALSE,
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkProgressBar:fraction: (attributes org.gtk.Property.get=gtk_progress_bar_get_fraction org.gtk.Property.set=gtk_progress_bar_set_fraction)
+   * GtkProgressBar:fraction:
    *
    * The fraction of total work that has been completed.
    */
   progress_props[PROP_FRACTION] =
-      g_param_spec_double ("fraction",
-                           P_("Fraction"),
-                           P_("The fraction of total work that has been completed"),
+      g_param_spec_double ("fraction", NULL, NULL,
                            0.0, 1.0,
                            0.0,
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkProgressBar:pulse-step: (attributes org.gtk.Property.get=gtk_progress_bar_get_pulse_step org.gtk.Property.set=gtk_progress_bar_set_pulse_step)
+   * GtkProgressBar:pulse-step:
    *
    * The fraction of total progress to move the bounding block when pulsed.
    */
   progress_props[PROP_PULSE_STEP] =
-      g_param_spec_double ("pulse-step",
-                           P_("Pulse Step"),
-                           P_("The fraction of total progress to move the bouncing block when pulsed"),
+      g_param_spec_double ("pulse-step", NULL, NULL,
                            0.0, 1.0,
                            0.1,
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkProgressBar:text: (attributes org.gtk.Property.get=gtk_progress_bar_get_text org.gtk.Property.set=gtk_progress_bar_set_text)
+   * GtkProgressBar:text:
    *
    * Text to be displayed in the progress bar.
    */
   progress_props[PROP_TEXT] =
-      g_param_spec_string ("text",
-                           P_("Text"),
-                           P_("Text to be displayed in the progress bar"),
+      g_param_spec_string ("text", NULL, NULL,
                            NULL,
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkProgressBar:show-text: (attributes org.gtk.Property.get=gtk_progress_bar_get_show_text org.gtk.Property.set=gtk_progress_bar_set_show_text)
+   * GtkProgressBar:show-text:
    *
    * Sets whether the progress bar will show a text in addition
    * to the bar itself.
@@ -243,14 +241,12 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
    * to %TRUE and [property@Gtk.ProgressBar:text] to the empty string (not %NULL).
    */
   progress_props[PROP_SHOW_TEXT] =
-      g_param_spec_boolean ("show-text",
-                            P_("Show text"),
-                            P_("Whether the progress is shown as text."),
+      g_param_spec_boolean ("show-text", NULL, NULL,
                             FALSE,
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkProgressBar:ellipsize: (attributes org.gtk.Property.get=gtk_progress_bar_get_ellipsize org.gtk.Property.set=gtk_progress_bar_set_ellipsize)
+   * GtkProgressBar:ellipsize:
    *
    * The preferred place to ellipsize the string.
    *
@@ -263,10 +259,7 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
    * progress bar's width is [method@Gtk.Widget.set_size_request].
    */
   progress_props[PROP_ELLIPSIZE] =
-      g_param_spec_enum ("ellipsize",
-                         P_("Ellipsize"),
-                         P_("The preferred place to ellipsize the string, if the progress bar "
-                            "does not have enough room to display the entire string, if at all."),
+      g_param_spec_enum ("ellipsize", NULL, NULL,
                          PANGO_TYPE_ELLIPSIZE_MODE,
                          PANGO_ELLIPSIZE_NONE,
                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
@@ -449,7 +442,24 @@ allocate_trough (GtkGizmo *gizmo,
     }
 
   gtk_widget_size_allocate (pbar->progress_widget, &alloc, -1);
+}
 
+static void
+snapshot_trough (GtkGizmo    *gizmo,
+                 GtkSnapshot *snapshot)
+
+{
+  GtkWidget *widget = gtk_widget_get_parent (GTK_WIDGET (gizmo));
+  GtkProgressBar *pbar = GTK_PROGRESS_BAR (widget);
+
+  if (pbar->progress_widget)
+    {
+      GtkCssBoxes boxes;
+      gtk_css_boxes_init (&boxes, GTK_WIDGET (gizmo));
+      gtk_snapshot_push_rounded_clip (snapshot, gtk_css_boxes_get_border_box (&boxes));
+      gtk_widget_snapshot_child (GTK_WIDGET (gizmo), pbar->progress_widget, snapshot);
+      gtk_snapshot_pop (snapshot);
+    }
 }
 
 static void
@@ -470,7 +480,7 @@ gtk_progress_bar_init (GtkProgressBar *pbar)
                                                  GTK_ACCESSIBLE_ROLE_NONE,
                                                  NULL,
                                                  allocate_trough,
-                                                 NULL,
+                                                 snapshot_trough,
                                                  NULL,
                                                  NULL, NULL);
   gtk_widget_set_parent (pbar->trough_widget, GTK_WIDGET (pbar));
@@ -749,7 +759,7 @@ gtk_progress_bar_set_activity_mode (GtkProgressBar *pbar,
 }
 
 /**
- * gtk_progress_bar_set_fraction: (attributes org.gtk.Method.set_property=fraction)
+ * gtk_progress_bar_set_fraction:
  * @pbar: a `GtkProgressBar`
  * @fraction: fraction of the task that’s been completed
  *
@@ -833,7 +843,7 @@ gtk_progress_bar_pulse (GtkProgressBar *pbar)
 }
 
 /**
- * gtk_progress_bar_set_text: (attributes org.gtk.Method.set_property=text)
+ * gtk_progress_bar_set_text:
  * @pbar: a `GtkProgressBar`
  * @text: (nullable): a UTF-8 string
  *
@@ -869,7 +879,7 @@ gtk_progress_bar_set_text (GtkProgressBar *pbar,
 }
 
 /**
- * gtk_progress_bar_set_show_text: (attributes org.gtk.Method.set_property=show-text)
+ * gtk_progress_bar_set_show_text:
  * @pbar: a `GtkProgressBar`
  * @show_text: whether to show text
  *
@@ -919,7 +929,7 @@ gtk_progress_bar_set_show_text (GtkProgressBar *pbar,
 }
 
 /**
- * gtk_progress_bar_get_show_text: (attributes org.gtk.Method.get_property=show-text)
+ * gtk_progress_bar_get_show_text:
  * @pbar: a `GtkProgressBar`
  *
  * Returns whether the `GtkProgressBar` shows text.
@@ -937,7 +947,7 @@ gtk_progress_bar_get_show_text (GtkProgressBar *pbar)
 }
 
 /**
- * gtk_progress_bar_set_pulse_step: (attributes org.gtk.Method.set_property=pulse-step)
+ * gtk_progress_bar_set_pulse_step:
  * @pbar: a `GtkProgressBar`
  * @fraction: fraction between 0.0 and 1.0
  *
@@ -1005,7 +1015,7 @@ gtk_progress_bar_set_orientation (GtkProgressBar *pbar,
 }
 
 /**
- * gtk_progress_bar_set_inverted: (attributes org.gtk.Method.set_property=inverted)
+ * gtk_progress_bar_set_inverted:
  * @pbar: a `GtkProgressBar`
  * @inverted: %TRUE to invert the progress bar
  *
@@ -1032,7 +1042,7 @@ gtk_progress_bar_set_inverted (GtkProgressBar *pbar,
 }
 
 /**
- * gtk_progress_bar_get_text: (attributes org.gtk.Method.get_property=text)
+ * gtk_progress_bar_get_text:
  * @pbar: a `GtkProgressBar`
  *
  * Retrieves the text that is displayed with the progress bar.
@@ -1051,7 +1061,7 @@ gtk_progress_bar_get_text (GtkProgressBar *pbar)
 }
 
 /**
- * gtk_progress_bar_get_fraction: (attributes org.gtk.Method.get_property=fraction)
+ * gtk_progress_bar_get_fraction:
  * @pbar: a `GtkProgressBar`
  *
  * Returns the current fraction of the task that’s been completed.
@@ -1067,7 +1077,7 @@ gtk_progress_bar_get_fraction (GtkProgressBar *pbar)
 }
 
 /**
- * gtk_progress_bar_get_pulse_step: (attributes org.gtk.Method.get_property=pulse-step)
+ * gtk_progress_bar_get_pulse_step:
  * @pbar: a `GtkProgressBar`
  *
  * Retrieves the pulse step.
@@ -1085,7 +1095,7 @@ gtk_progress_bar_get_pulse_step (GtkProgressBar *pbar)
 }
 
 /**
- * gtk_progress_bar_get_inverted: (attributes org.gtk.Method.get_property=inverted)
+ * gtk_progress_bar_get_inverted:
  * @pbar: a `GtkProgressBar`
  *
  * Returns whether the progress bar is inverted.
@@ -1101,7 +1111,7 @@ gtk_progress_bar_get_inverted (GtkProgressBar *pbar)
 }
 
 /**
- * gtk_progress_bar_set_ellipsize: (attributes org.gtk.Method.set_property=ellipsize)
+ * gtk_progress_bar_set_ellipsize:
  * @pbar: a `GtkProgressBar`
  * @mode: a `PangoEllipsizeMode`
  *
@@ -1130,7 +1140,7 @@ gtk_progress_bar_set_ellipsize (GtkProgressBar     *pbar,
 }
 
 /**
- * gtk_progress_bar_get_ellipsize: (attributes org.gtk.Method.get_property=ellipsize)
+ * gtk_progress_bar_get_ellipsize:
  * @pbar: a `GtkProgressBar`
  *
  * Returns the ellipsizing position of the progress bar.
