@@ -37,6 +37,9 @@
 #include "gtksnapshotprivate.h"
 
 #include "gdk/gdkhslaprivate.h"
+#include "gdk/gdkrgbaprivate.h"
+#include "gdk/gdkcolorprivate.h"
+#include "gdk/gdkcairoprivate.h"
 #include "gsk/gskroundedrectprivate.h"
 #include "gsk/gskrectprivate.h"
 
@@ -335,27 +338,30 @@ static void
 snapshot_frame_fill (GtkSnapshot          *snapshot,
                      const GskRoundedRect *outline,
                      const float           border_width[4],
-                     const GdkRGBA         colors[4],
+                     const GdkColor        colors[4],
                      guint                 hidden_side)
 {
   if (hidden_side)
     {
-      GdkRGBA real_colors[4];
+      GdkColor real_colors[4];
       guint i;
 
       for (i = 0; i < 4; i++)
         {
           if (hidden_side & (1 << i))
-            real_colors[i] = (GdkRGBA) { 0, 0, 0, 0 };
+            gdk_color_init_from_rgba (&real_colors[i], &GDK_RGBA_TRANSPARENT);
           else
-            real_colors[i] = colors[i];
+            gdk_color_init_copy (&real_colors[i], &colors[i]);
         }
 
       snapshot_frame_fill (snapshot, outline, border_width, real_colors, 0);
+
+      for (i = 0; i < 4; i++)
+        gdk_color_finish (&real_colors[i]);
       return;
     }
 
-  gtk_snapshot_append_border (snapshot, outline, border_width, colors);
+  gtk_snapshot_append_border2 (snapshot, outline, border_width, colors);
 }
 
 static void
@@ -409,7 +415,7 @@ static void
 render_frame_stroke (cairo_t              *cr,
                      const GskRoundedRect *border_box,
                      const double          border_width[4],
-                     GdkRGBA               colors[4],
+                     const GdkColor        colors[4],
                      guint                 hidden_side,
                      GtkBorderStyle        stroke_style)
 {
@@ -417,9 +423,9 @@ render_frame_stroke (cairo_t              *cr,
   GskRoundedRect stroke_box;
   guint i;
 
-  different_colors = !gdk_rgba_equal (&colors[0], &colors[1]) ||
-                     !gdk_rgba_equal (&colors[0], &colors[2]) ||
-                     !gdk_rgba_equal (&colors[0], &colors[3]);
+  different_colors = !gdk_color_equal (&colors[0], &colors[1]) ||
+                     !gdk_color_equal (&colors[0], &colors[2]) ||
+                     !gdk_color_equal (&colors[0], &colors[3]);
   different_borders = border_width[0] != border_width[1] ||
                       border_width[0] != border_width[2] ||
                       border_width[0] != border_width[3] ;
@@ -436,14 +442,15 @@ render_frame_stroke (cairo_t              *cr,
       double length = 0;
 
       /* FAST PATH:
-       * Mostly expected to trigger for focus rectangles */
-      for (i = 0; i < 4; i++) 
+       * Mostly expected to trigger for focus rectangles
+       */
+      for (i = 0; i < 4; i++)
         {
           length += _gtk_rounded_box_guess_length (&stroke_box, i);
         }
 
       gsk_rounded_rect_path (&stroke_box, cr);
-      gdk_cairo_set_source_rgba (cr, &colors[0]);
+      gdk_cairo_set_source_color (cr, GDK_COLOR_STATE_SRGB, &colors[0]);
       set_stroke_style (cr, border_width[0], stroke_style, length);
       cairo_stroke (cr);
     }
@@ -480,7 +487,7 @@ render_frame_stroke (cairo_t              *cr,
 
           _gtk_rounded_box_path_side (&stroke_box, cr, i);
 
-          gdk_cairo_set_source_rgba (cr, &colors[i]);
+          gdk_cairo_set_source_color (cr, GDK_COLOR_STATE_SRGB, &colors[i]);
           set_stroke_style (cr,
                             border_width[i],
                             stroke_style,
@@ -496,36 +503,38 @@ static void
 snapshot_frame_stroke (GtkSnapshot          *snapshot,
                        const GskRoundedRect *outline,
                        const float           border_width[4],
-                       GdkRGBA               colors[4],
+                       const GdkColor        colors[4],
                        guint                 hidden_side,
                        GtkBorderStyle        stroke_style)
 {
   double double_width[4] = { border_width[0], border_width[1], border_width[2], border_width[3] };
   cairo_t *cr;
 
-  cr = gtk_snapshot_append_cairo (snapshot,
-                                  &outline->bounds);
+  cr = gtk_snapshot_append_cairo (snapshot, &outline->bounds);
   render_frame_stroke (cr, outline, double_width, colors, hidden_side, stroke_style);
   cairo_destroy (cr);
 }
 
 static void
-color_shade (const GdkRGBA *color,
-             double         factor,
-             GdkRGBA       *color_return)
+color_shade (const GdkColor *color,
+             double          factor,
+             GdkColor       *color_return)
 {
+  GdkRGBA rgba;
   GdkHSLA hsla;
 
-  _gdk_hsla_init_from_rgba (&hsla, color);
+  gdk_color_to_float (color, GDK_COLOR_STATE_SRGB, (float *) &rgba);
+  _gdk_hsla_init_from_rgba (&hsla, &rgba);
   _gdk_hsla_shade (&hsla, &hsla, factor);
-  _gdk_rgba_init_from_hsla (color_return, &hsla);
+  _gdk_rgba_init_from_hsla (&rgba, &hsla);
+  gdk_color_from_rgba (color_return, color->color_state, &rgba);
 }
 
 static void
 snapshot_border (GtkSnapshot          *snapshot,
                  const GskRoundedRect *border_box,
                  const float           border_width[4],
-                 GdkRGBA               colors[4],
+                 GdkColor              colors[4],
                  GtkBorderStyle        border_style[4])
 {
   guint hidden_side = 0;
@@ -585,12 +594,12 @@ snapshot_border (GtkSnapshot          *snapshot,
                   hidden_side |= (1 << j);
                 else
                   dont_draw |= (1 << j);
-                
+
                 other_border[j] = border_width[j] / 3;
               }
-            
+
             snapshot_frame_fill (snapshot, border_box, other_border, colors, dont_draw);
-            
+
             other_box = *border_box;
             gsk_rounded_rect_shrink (&other_box,
                                      2 * other_border[GTK_CSS_TOP],
@@ -604,7 +613,7 @@ snapshot_border (GtkSnapshot          *snapshot,
         case GTK_BORDER_STYLE_RIDGE:
           {
             GskRoundedRect other_box;
-            GdkRGBA other_colors[4];
+            GdkColor other_colors[4];
             guint dont_draw = hidden_side;
             float other_border[4];
 
@@ -622,9 +631,9 @@ snapshot_border (GtkSnapshot          *snapshot,
                   dont_draw |= (1 << j);
                 other_border[j] = border_width[j] / 2;
               }
-            
+
             snapshot_frame_fill (snapshot, border_box, other_border, colors, dont_draw);
-            
+
             other_box = *border_box;
             gsk_rounded_rect_shrink (&other_box,
                                      other_border[GTK_CSS_TOP],
@@ -639,7 +648,7 @@ snapshot_border (GtkSnapshot          *snapshot,
           break;
         }
     }
-  
+
   snapshot_frame_fill (snapshot, border_box, border_width, colors, hidden_side);
 }
 
@@ -675,7 +684,7 @@ gtk_css_style_snapshot_border (GtkCssBoxes *boxes,
   else
     {
       GtkBorderStyle border_style[4];
-      GdkRGBA colors[4];
+      GdkColor colors[4];
       graphene_simd4f_t alpha_test_vector;
 
       /* Optimize the most common case of "This widget has no border" */
@@ -683,46 +692,54 @@ gtk_css_style_snapshot_border (GtkCssBoxes *boxes,
                           gtk_css_boxes_get_padding_rect (boxes)))
         return;
 
-      colors[0] = *gtk_css_color_value_get_rgba (style->used->border_top_color);
-      colors[1] = *gtk_css_color_value_get_rgba (style->used->border_right_color);
-      colors[2] = *gtk_css_color_value_get_rgba (style->used->border_bottom_color);
-      colors[3] = *gtk_css_color_value_get_rgba (style->used->border_left_color);
+      gtk_css_color_to_color (gtk_css_color_value_get_color (style->used->border_top_color), &colors[0]);
+      gtk_css_color_to_color (gtk_css_color_value_get_color (style->used->border_right_color), &colors[1]);
+      gtk_css_color_to_color (gtk_css_color_value_get_color (style->used->border_bottom_color), &colors[2]);
+      gtk_css_color_to_color (gtk_css_color_value_get_color (style->used->border_left_color), &colors[3]);
 
-      alpha_test_vector = graphene_simd4f_init (colors[0].alpha, colors[1].alpha, colors[2].alpha, colors[3].alpha);
-      if (graphene_simd4f_is_zero4 (alpha_test_vector))
-        return;
-
-      border_style[0] = _gtk_css_border_style_value_get (style->border->border_top_style);
-      border_style[1] = _gtk_css_border_style_value_get (style->border->border_right_style);
-      border_style[2] = _gtk_css_border_style_value_get (style->border->border_bottom_style);
-      border_style[3] = _gtk_css_border_style_value_get (style->border->border_left_style);
-
-      border_width[0] = gtk_css_number_value_get (style->border->border_top_width, 100);
-      border_width[1] = gtk_css_number_value_get (style->border->border_right_width, 100);
-      border_width[2] = gtk_css_number_value_get (style->border->border_bottom_width, 100);
-      border_width[3] = gtk_css_number_value_get (style->border->border_left_width, 100);
-
-      gtk_snapshot_push_debug (snapshot, "CSS border");
-      if (border_style[0] <= GTK_BORDER_STYLE_SOLID &&
-          border_style[1] <= GTK_BORDER_STYLE_SOLID &&
-          border_style[2] <= GTK_BORDER_STYLE_SOLID &&
-          border_style[3] <= GTK_BORDER_STYLE_SOLID)
+      alpha_test_vector = graphene_simd4f_init (colors[0].alpha,
+                                                colors[1].alpha,
+                                                colors[2].alpha,
+                                                colors[3].alpha);
+      if (!graphene_simd4f_is_zero4 (alpha_test_vector))
         {
-          /* The most common case of a solid border */
-          gtk_snapshot_append_border (snapshot,
-                                      gtk_css_boxes_get_border_box (boxes),
-                                      border_width,
-                                      colors);
+          border_style[0] = _gtk_css_border_style_value_get (style->border->border_top_style);
+          border_style[1] = _gtk_css_border_style_value_get (style->border->border_right_style);
+          border_style[2] = _gtk_css_border_style_value_get (style->border->border_bottom_style);
+          border_style[3] = _gtk_css_border_style_value_get (style->border->border_left_style);
+
+          border_width[0] = gtk_css_number_value_get (style->border->border_top_width, 100);
+          border_width[1] = gtk_css_number_value_get (style->border->border_right_width, 100);
+          border_width[2] = gtk_css_number_value_get (style->border->border_bottom_width, 100);
+          border_width[3] = gtk_css_number_value_get (style->border->border_left_width, 100);
+
+          gtk_snapshot_push_debug (snapshot, "CSS border");
+          if (border_style[0] <= GTK_BORDER_STYLE_SOLID &&
+              border_style[1] <= GTK_BORDER_STYLE_SOLID &&
+              border_style[2] <= GTK_BORDER_STYLE_SOLID &&
+              border_style[3] <= GTK_BORDER_STYLE_SOLID)
+            {
+              /* The most common case of a solid border */
+              gtk_snapshot_append_border2 (snapshot,
+                                           gtk_css_boxes_get_border_box (boxes),
+                                           border_width,
+                                           colors);
+            }
+          else
+            {
+              snapshot_border (snapshot,
+                               gtk_css_boxes_get_border_box (boxes),
+                               border_width,
+                               colors,
+                               border_style);
+            }
+           gtk_snapshot_pop (snapshot);
         }
-      else
-        {
-          snapshot_border (snapshot,
-                           gtk_css_boxes_get_border_box (boxes),
-                           border_width,
-                           colors,
-                           border_style);
-        }
-       gtk_snapshot_pop (snapshot);
+
+      gdk_color_finish (&colors[0]);
+      gdk_color_finish (&colors[1]);
+      gdk_color_finish (&colors[2]);
+      gdk_color_finish (&colors[3]);
     }
 }
 
@@ -733,17 +750,20 @@ gtk_css_style_snapshot_outline (GtkCssBoxes *boxes,
   GtkCssStyle *style = boxes->style;
   GtkBorderStyle border_style[4];
   float border_width[4];
-  GdkRGBA colors[4];
+  GdkColor colors[4];
 
   border_style[0] = _gtk_css_border_style_value_get (style->outline->outline_style);
   if (border_style[0] != GTK_BORDER_STYLE_NONE)
     {
-      const GdkRGBA *color;
+      GdkColor color;
 
-      color = gtk_css_color_value_get_rgba (style->used->outline_color);
+      gtk_css_color_to_color (gtk_css_color_value_get_color (style->used->outline_color), &color);
 
-      if (gdk_rgba_is_clear (color))
-        return;
+      if (gdk_color_is_clear (&color))
+        {
+          gdk_color_finish (&color);
+          return;
+        }
 
       border_width[0] = gtk_css_number_value_get (style->outline->outline_width, 100);
 
@@ -752,7 +772,7 @@ gtk_css_style_snapshot_outline (GtkCssBoxes *boxes,
 
       border_style[1] = border_style[2] = border_style[3] = border_style[0];
       border_width[3] = border_width[2] = border_width[1] = border_width[0];
-      colors[0] = colors[1] = colors[2] = colors[3] = *color;
+      colors[0] = colors[1] = colors[2] = colors[3] = color;
 
       gtk_snapshot_push_debug (snapshot, "CSS outline");
       snapshot_border (snapshot,
@@ -761,5 +781,6 @@ gtk_css_style_snapshot_outline (GtkCssBoxes *boxes,
                        colors,
                        border_style);
       gtk_snapshot_pop (snapshot);
+      gdk_color_finish (&color);
     }
 }
